from u3driver.commands.base_command import BaseCommand
class Tap(BaseCommand):
    def __init__(self, socket,request_separator,request_end,alt_object):
        super(Tap, self).__init__(socket,request_separator,request_end)
        self.alt_object=alt_object
    
    def execute(self):
        data=self.send_data(self.create_command('tapObject',self.alt_object))
        return self.handle_errors(data)



from u3driver.commands.base_command import BaseCommand


class GetUnityVersion(BaseCommand):
    def __init__(self, socket,request_separator,request_end):
        super(GetUnityVersion, self).__init__(socket,request_separator,request_end)
    
    def execute(self):
        data=self.send_data(self.create_command('getUnityVersion'))
        return data



import json
import threading
import time

from u3driver.commands.base_command import BaseCommand


class RecordProfileThread(threading.Thread):
    def __init__(self, target_func,  *args, **kwargs):
        super(RecordProfileThread, self).__init__(*args, **kwargs)
        self._stop_event = threading.Event()
        self.target_func = target_func
        self.is_record = True

    def run(self):
        data = self.target_func()
        if data == "record profile stop":
            self.is_record = False
            self.stop()

    def stop(self):
        self._stop_event.set()

    def stopped(self):
        return self._stop_event.is_set()


class RecordProfile(BaseCommand):
    '''
    record = 0 停止打点
    record = 1 开始打点
    '''
    def __init__(self, socket,request_separator,request_end,record = "1"):
        super(RecordProfile, self).__init__(socket,request_separator,request_end)
        self.record = record
        self.stop_record = False
        self.record_files = []




    def get_data(self):
        while True:
            if self.stop_record:
                time.sleep(0.1)
                return

            data = self.recvall()
            
            if data == "record profile stop":
                time.sleep(0.1)
                return data
            
            if data == "":
                time.sleep(0.1)
                continue
            
            data = json.loads(data)
            self.record_files.append(data)
            

    def start(self):
        data = self.send_data(self.create_command('RecordProfile','1'))
        
        self.thread = RecordProfileThread(target_func=self.get_data)
        self.thread.start()
        # print(data)
        return data
    
    def stop(self):
        self.stop_recvall()
        self.stop_record = True
        self.thread.stop()
        data = self.send_data(self.create_command('RecordProfile','0'))

        while data != "record profile stop":
            data = json.loads(data)
            self.record_files.append(data)
            data = self.recvall()

        time.sleep(0.5)
        self.clear_recv()
        # print(data)
        return data
    
    def check(self):
        if len(self.record_files) > 0:
            ret = self.record_files[::]
            self.record_files = []
            return ret
        return []

        


class AltUnityException(Exception):
    def __init__(self,message):
        super(AltUnityException,self).__init__(message)
           
class NotFoundException(AltUnityException):
    def __init__(self,message):
        super(NotFoundException,self).__init__(message)

class PropertyNotFoundException(AltUnityException):
    def __init__(self,message):
        super(PropertyNotFoundException,self).__init__(message)

class MethodNotFoundException(AltUnityException):
    def __init__(self,message):
        super(MethodNotFoundException,self).__init__(message)

class ComponentNotFoundException(AltUnityException):
    def __init__(self,message):
        super(ComponentNotFoundException,self).__init__(message)

class CouldNotPerformOperationException(AltUnityException):
    def __init__(self,message):
        super(CouldNotPerformOperationException,self).__init__(message)

class IncorrectNumberOfParametersException(AltUnityException):
    def __init__(self,message):
        super(IncorrectNumberOfParametersException,self).__init__(message)

class CouldNotParseJsonStringException(AltUnityException):
    def __init__(self,message):
        super(CouldNotParseJsonStringException,self).__init__(message)

class FailedToParseArgumentsException(AltUnityException):
    def __init__(self,message):
        super(FailedToParseArgumentsException,self).__init__(message)

class ObjectWasNotFoundException(AltUnityException):
    def __init__(self,message):
        super(ObjectWasNotFoundException,self).__init__(message)

class PropertyCannotBeSetException(AltUnityException):
    def __init__(self,message):
        super(PropertyCannotBeSetException,self).__init__(message)

class NullReferenceException(AltUnityException):
    def __init__(self,message):
        super(NullReferenceException,self).__init__(message)

class UnknownErrorException(AltUnityException):
    def __init__(self,message):
        super(UnknownErrorException,self).__init__(message)

class FormatException(AltUnityException):
    def __init__(self,message):
        super(FormatException,self).__init__(message)

class WaitTimeOutException(AltUnityException):
    def __init__(self,message):
        super(WaitTimeOutException,self).__init__(message)



from u3driver.commands.base_command import BaseCommand

class SetText(BaseCommand):
    def __init__(self, socket,request_separator,request_end, text_value, alt_object):
        super(SetText, self).__init__(socket,request_separator,request_end)
        self.alt_object = alt_object
        self.text = text_value

    def execute(self):
        data = self.send_data(self.create_command('setText', self.alt_object, self.text))
        return self.handle_errors(data)



import json

from u3driver.commands.base_command import BaseCommand


class Drag(BaseCommand):
    def __init__(self, socket,request_separator,request_end,path,x1,y1,x2 = None,y2 = None):
        super(Drag, self).__init__(socket,request_separator,request_end)
        self.path = path
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2
    
    def execute(self):
        #x1,y1:[0-1] 
        screen = self.send_data(self.create_command('getScreen'))
        screen = json.loads(screen)
        screen = {"width":int(screen["width"]), "height":int(screen['height'])}


        if self.x2 == None or self.y2 == None:
            data = self.send_data(self.create_command('dragObject', self.path,self.x1*screen["width"],self.y1*screen["height"]))
        else:
            data = self.send_data(self.create_command('dragObject', self.path,self.x1*screen["width"],self.y1*screen["height"],self.x2*screen["width"],self.y2*screen["height"]))
        return data



import json

from u3driver.commands.command_returning_alt_elements import \
    CommandReturningAltElements


class FindText(CommandReturningAltElements):
    def __init__(self, socket,request_separator,request_end,appium_driver,keyword):
        super(FindText, self).__init__(socket,request_separator,request_end,appium_driver)
        self.keyword = keyword
    
    def execute(self):
        data = self.send_data(self.create_command('findText',self.keyword))
        return data



from u3driver.commands.base_command import BaseCommand
class GetText(BaseCommand):
    def __init__(self, socket,request_separator,request_end,alt_object):
        super(GetText, self).__init__(socket,request_separator,request_end)
        self.alt_object=alt_object

    def execute(self):
        data = self.send_data(self.create_command('getText', self.alt_object))
        return self.handle_errors(data)



import json

from u3driver.altElement import AltElement
from u3driver.commands.base_command import BaseCommand

BUFFER_SIZE = 1024
class CommandReturningAltElements(BaseCommand):
    def __init__(self, socket,request_separator,request_end,appium_driver):
        self.request_separator=request_separator
        self.request_end=request_end
        self.socket=socket
        self.appium_driver=appium_driver

    def get_alt_element(self, data):
        print(data)
        if (data != '' and 'error:' not in data):
            alt_el = None
            try:
                alt_el = AltElement(self, self.appium_driver, data)
            except:
                alt_el = AltElement(self, None, data)
            print('Element ' + alt_el.name + ' found at x:' + str(alt_el.x) + ' y:' + str(alt_el.y) + ' mobileY:' + str(alt_el.mobileY))
            return alt_el
        self.handle_errors(data)
        return None
    
    def get_alt_elements(self, data):
        if (data != '' and 'error:' not in data):
            alt_elements = []
            elements = []
            try:
                elements = json.loads(data)
            except:
                print("exception !!!!!!")
                raise Exception("Couldn't parse json data: " + data)
            
            alt_el = None
            for i in range(0, len(elements)):
                try:
                    alt_el = AltElement(self, self.appium_driver, json.dumps(elements[i]))
                except:
                    alt_el = AltElement(self, None, json.dumps(elements[i]))
                    
                alt_elements.append(alt_el)
                print('Element ' + alt_el.name + ' found at x:' + str(alt_el.x) + ' y:' + str(alt_el.y) + ' mobileY:' + str(alt_el.mobileY))
            return alt_elements
            
        self.handle_errors(data)
        return None



from u3driver.commands.base_command import BaseCommand
class Find(BaseCommand):
    def __init__(self, socket,request_separator,request_end,alt_object,value):
        super(Find, self).__init__(socket,request_separator,request_end)
        self.alt_object=alt_object
        self.value = value
    
    def execute(self):
        data=self.send_data(self.create_command('objectFind',self.alt_object,'name', self.value))
        return data



from u3driver.commands.base_command import BaseCommand


class EnableLogging(BaseCommand):

    def __init__(self, socket,request_separator,request_end,log_flag):
        super(EnableLogging, self).__init__(socket,request_separator,request_end)
        self.log_flag=log_flag
    
    def execute(self):
        if(self.log_flag):
            self.send_data(self.create_command('enableLogging','true'))
        else:
            self.send_data(self.create_command('enableLogging','false')) 


from u3driver.commands.ObjectCommands.child_index import *
from u3driver.commands.ObjectCommands.drag import *
from u3driver.commands.ObjectCommands.find import *
from u3driver.commands.ObjectCommands.get_text import *
from u3driver.commands.ObjectCommands.parent import *
from u3driver.commands.ObjectCommands.set_text import *
from u3driver.commands.ObjectCommands.tap import *



# coding: utf-8
import os, re
from setuptools import setup, find_namespace_packages

version = '1.0.0'
appname = "u3driver"
with open(os.path.join("u3driver", "__init__.py"), encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)
    # for github action
    tag = os.getenv('tag')
    build_number = os.getenv('build_number')
    print(f'build_number={build_number}')
    if tag and tag.startswith("refs/tags/v"):
        version = tag.replace("refs/tags/v", "")
    elif build_number:
        version = version.replace("x", build_number)
    else:
        # add version
        x_y_z = [int(x) for x in version.split('.')]
        x_y_z[-1] += 1
        version = '.'.join(str(x) for x in x_y_z)

setup(
    name=appname,
    version=version,
    python_requires='>=3.6',
    description='u3driver',
    url='https://github.com/king3soft/u3driver',
    author='king3soft',
    author_email='buutuud@gmail.com',
    license='GPLv3',
    include_package_data=True,
    packages=find_namespace_packages(include=['u3driver.*', "u3driver"]),
    install_requires=''''''.split('\n'),
    zip_safe=False)



import json
import threading
import time

from u3driver.commands.base_command import BaseCommand


class DebugModeThread(threading.Thread):
    def __init__(self, target_func,  *args, **kwargs):
        super(DebugModeThread, self).__init__(*args, **kwargs)
        self._stop_event = threading.Event()
        self.target_func = target_func
        self.is_record = True

    def run(self):
        data = self.target_func()
        if data == "Close Debug Mode":
            self.is_record = False
            # print("Debug Mode Closed!!!!!!!!!")
            self.stop()

    def stop(self):
        self._stop_event.set()

    def stopped(self):
        return self._stop_event.is_set()


class DebugMode(BaseCommand):

    def __init__(self, socket,request_separator,request_end,file_path = None):
        self.stop_record = False
        self.pause_record = False
        print("DebugMode.init")
        super().__init__(socket,request_separator,request_end)
        self.file_path = file_path

    def get_record(self):
        # self.stop_record = False
        # self.pause_record = False
        count = 0
        while True:
            if self.stop_record:
                # print("stop_record")
                time.sleep(0.1)
                return
            if self.pause_record:
                # print("pause_record")
                time.sleep(0.1)
                continue
            # print("get_record")
            data = self.recvall()
            self.data_str = ''
            if data == "Close Debug Mode":
                # print("Close Debug Mode")
                time.sleep(0.1)
                return data
            if data == "200":
                # print("200")
                time.sleep(0.1)
                continue
            if data == '':
                # print("empty")
                time.sleep(0.1)
                continue

            # print(f'data = {data}')
            count += 1
            json_data = json.loads(data)
            self.time = 2
            if "time" in json_data:
                self.time = float(json_data['time'])
            if "start position" in json_data:
                self.start_position = tuple(eval(json_data['start position']))
            elif "end position" in json_data:
                self.end_position = tuple(eval(json_data['end position']))
                if self.start_position != None and self.end_position != None:
                    self.data_str += '\t' * self.tap_count + f'time.sleep({self.time})\n'
                    self.data_str += '\t' * self.tap_count + 'udriver.drag_object("'+json_data['name'].replace("/","//")+f'",{self.start_position[0] / self.screen["width"]},{self.start_position[1] / self.screen["height"]},{self.end_position[0] / self.screen["width"]},{self.end_position[1] / self.screen["height"]})\n'
                    self.start_position = None
                    self.end_position = None
            elif "value" in json_data:
                self.data_str += '\t' * self.tap_count + f'time.sleep({self.time})\n'
                self.data_str += '\t' * self.tap_count + 'udriver.find_object(By.PATH,"'+json_data['name'].replace("/","//")+'")'
                self.data_str += '.set_text("' + json_data['value'] + '")\n'
            elif "name" in json_data:
                self.data_str += '\t' * self.tap_count + f'time.sleep({self.time})\n'
                self.data_str += '\t' * self.tap_count + 'udriver.find_object(By.PATH,"'+json_data['name'].replace("/","//")+'")'
                self.data_str += '.tap()\n'
            if self.file_path != None:
                    all_str = '' 
                    with open(file=self.file_path, mode='r+', encoding='utf-8') as f:
                        all_str = f.read()

                    all_str = all_str.replace(self.end_str,"")
                    all_str += self.data_str
                    all_str += self.end_str

                    with open(file=self.file_path, mode='w+', encoding='utf-8') as f:
                        
                        f.write(all_str)


    def is_record(self):
        return self.thread.is_record
    
    def stop(self):
        # 停掉线程后清空
        self.stop_recvall()
        self.stop_record = True
        self.thread.stop()
        self.send_data(self.create_command('stopDebugMode'))
        time.sleep(0.5)
        self.clear_recv()
        # time.sleep(0.5)
        # self.recvall()
        

    
    def pause(self):
        self.pause_record = True
        # self.thread.stop()
        # time.sleep(0.5)
        self.send_data(self.create_command('pauseDebugMode'))
        time.sleep(0.5)
        self.clear_recv()
        # self.recvall()
        # time.sleep(0.5)
        # self.stop_recvall()
    
    def resume(self):
        # self.clear_recv()
        self.pause_record = False
        # time.sleep(0.5)
        self.send_data(self.create_command('resumeDebugMode'))
        # self.recvall()
        # time.sleep(0.5)
        # self.stop_recvall()
        # self.thread.start()

    def sync_record(self):
        self.screen = self.send_data(self.create_command('getScreen'))
        print(self.screen)
        self.screen = json.loads(self.screen)
        self.screen = {"width":int(self.screen["width"]), "height":int(self.screen['height'])}

        response = self.send_data(self.create_command('debugMode', '0'))
        if response == "Open Debug Mode" or response == "Debug Already Opened":
            ##结束行有17行
            self.end_str = """\texcept Exception as e:\n\t\tprint(f'{e}')\n\t\traise e\n\nif __name__ == '__main__':\n\tparser = argparse.ArgumentParser()\n\tparser.add_argument('-s', help="device serial")\n\tparser.add_argument('-i', help="ip address")\n\tparser.add_argument('-port', help="ip address")\n\targs = parser.parse_args()\n\tdevice_s = args.s\n\tip = args.i\n\tport = int(args.port)\n\tudriver = AltrunUnityDriver(device_s,"", ip, TCP_PORT=port,timeout=60, log_flag=True)\n\tAutoRun(udriver)\n\tudriver.stop()"""
            self.tap_count = 0
            if self.file_path != None:
                self.data_str = ''
                self.data_str += 'from u3driver import AltrunUnityDriver\n'
                self.data_str += 'from u3driver import By\n'
                self.data_str += 'import time\n'
                self.data_str += 'import argparse\n\n'
                self.data_str += 'def AutoRun(udriver):\n'
                self.tap_count += 1
                self.data_str += '\t' * self.tap_count + 'try:\n'
                self.tap_count += 1
                
                if self.file_path != None:
                        with open(file=self.file_path, mode='w+', encoding='utf-8') as self.f:
                            self.f.write(self.data_str)
            self.start_position = None
            self.end_position = None
            while True:
                data = self.recvall()
                self.data_str = ''
                if data == "Close Debug Mode":
                    return data
                json_data = json.loads(data)
                time = 2
                if "time" in json_data:
                    time = float(json_data['time'])
                if "start position" in json_data:
                    self.start_position = tuple(eval(json_data['start position']))
                elif "end position" in json_data:
                    self.end_position = tuple(eval(json_data['end position']))
                    if self.start_position != None and self.end_position != None:
                        self.data_str += '\t' * self.tap_count + f'time.sleep({time})\n'
                        self.data_str += '\t' * self.tap_count + 'udriver.drag_object("'+json_data['name'].replace("/","//")+f'",{self.start_position[0] / self.screen["width"]},{self.start_position[1] / self.screen["height"]},{self.end_position[0] / self.screen["width"]},{self.end_position[1] / self.screen["height"]})\n'
                        self.start_position = None
                        self.end_position = None
                elif "value" in json_data:
                    self.data_str += '\t' * self.tap_count + f'time.sleep({time})\n'
                    self.data_str += '\t' * self.tap_count + 'udriver.find_object(By.PATH,"'+json_data['name'].replace("/","//")+'")'
                    self.data_str += '.set_text("' + json_data['value'] + '")\n'
                elif "name" in json_data:
                    self.data_str += '\t' * self.tap_count + f'time.sleep({time})\n'
                    self.data_str += '\t' * self.tap_count + 'udriver.find_object(By.PATH,"'+json_data['name'].replace("/","//")+'")'
                    self.data_str += '.tap()\n'
                if self.file_path != None:
                        all_str = '' 
                        with open(file=self.file_path, mode='r+', encoding='utf-8') as f:
                            all_str = f.read()

                        all_str = all_str.replace(self.end_str,"")
                        all_str += self.data_str
                        all_str += self.end_str

                        with open(file=self.file_path, mode='w+', encoding='utf-8') as f:
                            
                            f.write(all_str)
        return response
    
    def async_record(self):
        self.screen = self.send_data(self.create_command('getScreen'))
        print(self.screen)
        self.screen = json.loads(self.screen)
        self.screen = {"width":int(self.screen["width"]), "height":int(self.screen['height'])}

        response = self.send_data(self.create_command('debugMode', '0'))
        if response == "Open Debug Mode" or response == "Debug Already Opened":
            ##结束行有17行
            self.end_str = """\texcept Exception as e:\n\t\tprint(f'{e}')\n\t\traise e\n\nif __name__ == '__main__':\n\tparser = argparse.ArgumentParser()\n\tparser.add_argument('-s', help="device serial")\n\tparser.add_argument('-i', help="ip address")\n\tparser.add_argument('-port', help="ip address")\n\targs = parser.parse_args()\n\tdevice_s = args.s\n\tip = args.i\n\tport = int(args.port)\n\tudriver = AltrunUnityDriver(device_s,"", ip, TCP_PORT=port,timeout=60, log_flag=True)\n\tAutoRun(udriver)\n\tudriver.stop()"""
            self.tap_count = 0
            if self.file_path != None:
                self.data_str = ''
                self.data_str += 'from u3driver import AltrunUnityDriver\n'
                self.data_str += 'from u3driver import By\n'
                self.data_str += 'import time\n'
                self.data_str += 'import argparse\n\n'
                self.data_str += 'def AutoRun(udriver):\n'
                self.tap_count += 1
                self.data_str += '\t' * self.tap_count + 'try:\n'
                self.tap_count += 1
                
                if self.file_path != None:
                        with open(file=self.file_path, mode='w+', encoding='utf-8') as self.f:
                            self.f.write(self.data_str)
            self.start_position = None
            self.end_position = None

            self.thread = DebugModeThread(target_func=self.get_record)
            # self.thread = threading.Thread(target=self.get_record)
            self.thread.start()

                
        return response
